//============================================================================
//BubbleItem Class
//Copyright  2008 Andy Gill
//
//Based on orginal code Copyright  John Champion
//
//This library is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public
//License as published by the Free Software Foundation; either
//version 2.1 of the License, or (at your option) any later version.
//
//This library is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.
//
//You should have received a copy of the GNU Lesser General Public
//License along with this library; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//=============================================================================


#region Using directives

using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Runtime.Serialization;
using System.Security.Permissions;

#endregion

namespace ZedGraph
{
	/// <summary>
	/// Encapsulates an Bubble curve type that displays a bubble in which 
	/// the size and position are set by data values from the
    /// <see cref="PointPair"/> struct.
	/// </summary>
	/// <remarks>The <see cref="BubbleItem"/> type is intended for displaying
	/// bubbles that represent values by position and volume by area.  
    /// The position of each bubble is set according to the 
    /// <see cref="PointPair"/> values.  The independent axis is assigned with 
    /// <see cref="BubbleSettings.Base"/>, and is a <see cref="BubbleBase"/> 
    /// enum type.
    /// The size of the beubble is set according the Z value in the 
    /// <see cref="PointPair"/> struct.</remarks>
	/// <author> Andy Gill </author>
	/// <version> $Revision: 1.0 $ $Date: 2008/10/016 10:00:00 $ </version>
	[Serializable]
	public class BubbleItem : CurveItem, ICloneable, ISerializable
    {
    #region Fields
        /// <summary>
        /// Private field that stores a reference to the <see cref="ZedGraph.Bubble"/>
        /// class defined for this <see cref="BubbleItem"/>.  Use the public
        /// property Bubble to access this value.
        /// </summary>
        protected Bubble _bubble;
    #endregion

    #region Properties
        /// <summary>
        /// Gets a reference to the <see cref="ZedGraph.Bubble"/> class defined
        /// for this <see cref="BubbleItem"/>.
        /// </summary>
        public Bubble Bubble
        {
            get { return _bubble; }
        }

        /// <summary>
        /// Gets a flag indicating if the Z data range should be included in the axis scaling calculations.
        /// </summary>
        /// <param name="pane">The parent <see cref="GraphPane" /> of this <see cref="CurveItem" />.
        /// </param>
        /// <value>true if the Z data are included, false otherwise</value>
        override internal bool IsZIncluded(GraphPane pane)
        {
            return this is BubbleItem;
        }

        /// <summary>
        /// Gets a flag indicating if the X axis is the independent axis for this <see cref="CurveItem" />
        /// </summary>
        /// <param name="pane">The parent <see cref="GraphPane" /> of this <see cref="CurveItem" />.
        /// </param>
        /// <value>true if the X axis is independent, false otherwise</value>
        override internal bool IsXIndependent(GraphPane pane)
        {
            return pane._bubbleSettings.Base == BubbleBase.X || pane._bubbleSettings.Base == BubbleBase.X2;
        }
    #endregion

    #region Methods
        /// <summary>
        /// Do all rendering associated with this <see cref="BubbleItem"/> to the specified
        /// <see cref="Graphics"/> device.  This method is normally only be called by the 
        /// Draw method of the parent <see cref="ZedGraph.CurveList"/> collection object.
        /// </summary>
        /// <param name="g">
        /// A graphic device object to be drawn into.  This is normally e.Graphics from the
        /// PaintEventArgs argument to the Paint() method.
        /// </param>
        /// <param name="pane">
        /// A reference to the <see cref="ZedGraph.GraphPane"/> object that is the parent or
        /// owner of this object.
        /// </param>
        /// <param name="pos">The ordinal position of the current <see cref="Bubble"/>
        /// curve.</param>
        /// <param name="scaleFactor">
        /// The scaling factor to be used for rendering objects.  This is calculated and
        /// passed down by the parent <see cref="ZedGraph.GraphPane"/> object using the
        /// <see cref="PaneBase.CalcScaleFactor"/> method, and is used to proportionally adjust
        /// bubble width, font sizes, etc. according to the actual size of the graph.
        /// </param>
        override public void Draw(Graphics g, GraphPane pane, int pos,
                                    float scaleFactor)
        {
            // Pass the drawing onto the bubble class
            if (_isVisible)
                _bubble.DrawBubbles(g, pane, this, BaseAxis(pane), ValueAxis(pane), pos, scaleFactor);
        }

        /// <summary>
        /// Draw a legend key entry for this <see cref="BubbleItem"/> at the specified location
        /// </summary>
        /// <param name="g">
        /// A graphic device object to be drawn into.  This is normally e.Graphics from the
        /// PaintEventArgs argument to the Paint() method.
        /// </param>
        /// <param name="pane">
        /// A reference to the <see cref="ZedGraph.GraphPane"/> object that is the parent or
        /// owner of this object.
        /// </param>
        /// <param name="rect">The <see cref="RectangleF"/> struct that specifies the
        /// location for the legend key</param>
        /// <param name="scaleFactor">
        /// The scaling factor to be used for rendering objects.  This is calculated and
        /// passed down by the parent <see cref="ZedGraph.GraphPane"/> object using the
        /// <see cref="PaneBase.CalcScaleFactor"/> method, and is used to proportionally adjust
        /// bubble width, font sizes, etc. according to the actual size of the graph.
        /// </param>
        override public void DrawLegendKey(Graphics g, GraphPane pane, RectangleF rect, float scaleFactor)
        {
            //Make the legand graphic round for bubbles
            rect.Width = rect.Height;
            _bubble.Draw(g, pane, rect, scaleFactor, true, false, null);
        }

        /// <summary>
        /// Determine the coords for the rectangle associated with a specified point for 
        /// this <see cref="CurveItem" />
        /// </summary>
        /// <param name="pane">The <see cref="GraphPane" /> to which this curve belongs</param>
        /// <param name="i">The index of the point of interest</param>
        /// <param name="coords">A list of coordinates that represents the "rect" for
        /// this point (used in an html AREA tag)</param>
        /// <returns>true if it's a valid point, false otherwise</returns>
        override public bool GetCoords(GraphPane pane, int i, out string coords)
        {
            coords = string.Empty;

            if (i < 0 || i >= _points.Count)
                return false;

            Axis valueAxis = ValueAxis(pane);
            Axis baseAxis = BaseAxis(pane);

            // curBase = the scale value on the base axis of the current bubble
            // curHiVal = the scale value on the value axis of the current bubble
            // curLowVal = the scale value of the bottom of the bubble
            double curBase, curLowVal, curHiVal;
            ValueHandler valueHandler = new ValueHandler(pane, false);
            valueHandler.GetValues(this, i, out curBase, out curLowVal, out curHiVal);

            // Any value set to double max is invalid and should be skipped
            // This is used for calculated values that are out of range, divide
            //   by zero, etc.
            // Also, any value <= zero on a log scale is invalid

            if (!_points[i].IsInvalid3D)
            {
                float width = (float)_points[i].Z;
                if (width != 0)
                {
                    //pixel value for the bubble rect X on the base axis
                    float left = baseAxis.Scale.Transform(_isOverrideOrdinal, i, curBase);
                    //pixel value for the bubble rect Y on the base axis
                    float top = valueAxis.Scale.Transform(_isOverrideOrdinal, i, curHiVal);

                    float scaledWidth = width * pane.CalcScaleFactor();
                    float x = left - (scaledWidth / 2);
                    float y = top - (scaledWidth / 2);
                    float w = x + scaledWidth;
                    float h = y + scaledWidth;

                    // Draw the bubble
                    if (baseAxis is XAxis || baseAxis is X2Axis)
                        coords = String.Format("{0:f0},{1:f0},{2:f0},{3:f0}", x, y, w, h);
                    else
                        coords = String.Format("{0:f0},{1:f0},{2:f0},{3:f0}", x, y, w, h);

                    return true;
                }
            }

            return false;
        }

        #endregion

	#region Constructors
		/// <summary>
        /// Create a new <see cref="BubbleItem"/> using the specified properties.
		/// </summary>
		/// <param name="label">The label that will appear in the legend.</param>
		/// <param name="x">An array of double precision values that define
		/// the independent (X axis) values for this curve</param>
		/// <param name="y">An array of double precision values that define
		/// the dependent (Y axis) values for this curve</param>
		/// <param name="baseVal">An array of double precision values that define the
		/// base value (the width) of the bubbles for this curve.
		/// </param>
		/// <param name="color">A <see cref="Color"/> value that will be applied to
        /// the <see cref="ZedGraph.Bubble.Fill"/> and <see cref="ZedGraph.Bubble.Border"/> properties.
		/// </param>
		public BubbleItem( string label, double[] x, double[] y, double[] baseVal, Color color ) :
			this( label, new PointPairList( x, y, baseVal ), color )
		{
		}
		
		/// <summary>
        /// Create a new <see cref="BubbleItem"/> using the specified properties.
		/// </summary>
		/// <param name="label">The label that will appear in the legend.</param>
		/// <param name="points">A <see cref="IPointList"/> of double precision value trio's that define
		/// the X, Y, and width values for this curve</param>
		/// <param name="color">A <see cref="Color"/> value that will be applied to
		/// the <see cref="ZedGraph.Bubble.Fill"/> and <see cref="ZedGraph.Bubble.Border"/> properties.
		/// </param>
		public BubbleItem( string label, IPointList points, Color color )
			: base( label, points )
		{
            _bubble = new Bubble(color);
		}

		/// <summary>
		/// The Copy Constructor
		/// </summary>
        /// <param name="rhs">The <see cref="BubbleItem"/> object from which to copy</param>
		public BubbleItem( BubbleItem rhs ) : base( rhs )
		{
            _bubble = rhs._bubble.Clone(); // new BubbleItem( rhs.Bubble );
		}

		/// <summary>
		/// Implement the <see cref="ICloneable" /> interface in a typesafe manner by just
		/// calling the typed version of <see cref="Clone" />
		/// </summary>
		/// <returns>A deep copy of this object</returns>
		object ICloneable.Clone()
		{
			return this.Clone();
		}

		/// <summary>
		/// Typesafe, deep-copy clone method.
		/// </summary>
		/// <returns>A new, independent copy of this class</returns>
        public BubbleItem Clone()
		{
			return new BubbleItem( this );
		}

	#endregion

	#region Serialization
		/// <summary>
		/// Current schema value that defines the version of the serialized file
		/// </summary>
		public const int schema3 = 1;

		/// <summary>
		/// Constructor for deserializing objects
		/// </summary>
		/// <param name="info">A <see cref="SerializationInfo"/> instance that defines the serialized data
		/// </param>
		/// <param name="context">A <see cref="StreamingContext"/> instance that contains the serialized data
		/// </param>
        protected BubbleItem(SerializationInfo info, StreamingContext context)
            : base(info, context)
		{
			// The schema value is just a file version parameter.  You can use it to make future versions
			// backwards compatible as new member variables are added to classes
			int sch = info.GetInt32( "schema3" );
		}
		/// <summary>
		/// Populates a <see cref="SerializationInfo"/> instance with the data needed to serialize the target object
		/// </summary>
		/// <param name="info">A <see cref="SerializationInfo"/> instance that defines the serialized data</param>
		/// <param name="context">A <see cref="StreamingContext"/> instance that contains the serialized data</param>
		[SecurityPermissionAttribute(SecurityAction.Demand,SerializationFormatter=true)]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData( info, context );
		}
	#endregion

	}
}
