//============================================================================
//BubbleSettings Class 
//Copyright  2008  Andy Gill
//
//Based on orginal code Copyright  John Champion
//
//This library is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public
//License as published by the Free Software Foundation; either
//version 2.1 of the License, or (at your option) any later version.
//
//This library is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.
//
//You should have received a copy of the GNU Lesser General Public
//License along with this library; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//=============================================================================

using System;
using System.Drawing;
using System.Text;
using System.Runtime.Serialization;
using System.Security.Permissions;

namespace ZedGraph
{
	/// <summary>
	/// Class that handles the global settings for bubble charts
	/// </summary>
	/// 
	/// <author> Andy Gill </author>
	/// <version> $Revision: 1.0 $ $Date: 2008/10/16 10:00:00 $ </version>
	[Serializable]
	public class BubbleSettings : ISerializable
	{
	#region Fields

		/// <summary>Private field that determines the base axis from which <see cref="Bubble"/>
		/// graphs will be displayed.  The base axis is the axis from which the bubbles grow with
		/// increasing value. The value is of the enumeration type <see cref="ZedGraph.BubbleBase"/>.
		/// To access this value, use the public property <see cref="Base"/>.
		/// </summary>
		/// <seealso cref="Default.Base"/>
		private BubbleBase _base;

		/// <summary>
		/// private field that stores the owner GraphPane that contains this BubbleSettings instance.
		/// </summary>
		internal GraphPane _ownerPane;

	#endregion

	#region Constructors

		/// <summary>
		/// Constructor to build a <see cref="BubbleSettings" /> instance from the defaults.
		/// </summary>
		public BubbleSettings( GraphPane parentPane )
		{
           	_base = Default.Base;
            _ownerPane = parentPane;
		}

		/// <summary>
		/// Copy constructor
		/// </summary>
		/// <param name="rhs">the <see cref="BubbleSettings" /> instance to be copied.</param>
		/// <param name="parentPane">The <see cref="GraphPane" /> that will be the
        /// parent of this new <see cref="BubbleSettings" /> object.</param>
		public BubbleSettings( BubbleSettings rhs, GraphPane parentPane )
		{
			_base = rhs._base;
			_ownerPane = parentPane;
		}

	#endregion

	#region Bubble Properties

		/// <summary>Determines the base axis from which <see cref="Bubble"/>
		/// graphs will be displayed.
		/// </summary>
		/// <remarks>The base axis is the axis from which the bubbles grow with
		/// increasing value. The value is of the enumeration type <see cref="ZedGraph.BubbleBase"/>.
		/// </remarks>
		/// <seealso cref="Default.Base"/>
		public BubbleBase Base
		{
			get { return _base; }
			set { _base = value; }
		}

    #endregion

	#region Serialization

		/// <summary>
		/// Current schema value that defines the version of the serialized file
		/// </summary>
		public const int schema = 1;

		/// <summary>
		/// Constructor for deserializing objects
		/// </summary>
		/// <remarks>
		/// You MUST set the _ownerPane property after deserializing a BubbleSettings object.
		/// </remarks>
		/// <param name="info">A <see cref="SerializationInfo"/> instance that defines the
		/// serialized data
		/// </param>
		/// <param name="context">A <see cref="StreamingContext"/> instance that contains
		/// the serialized data
		/// </param>
		internal BubbleSettings( SerializationInfo info, StreamingContext context )
		{
			// The schema value is just a file version parameter.  You can use it to make future versions
			// backwards compatible as new member variables are added to classes
			int sch = info.GetInt32( "schema" );

            _base = (BubbleBase)info.GetValue("base", typeof(BubbleBase));
		}

		/// <summary>
		/// Populates a <see cref="SerializationInfo"/> instance with the data needed to serialize the target object
		/// </summary>
		/// <param name="info">A <see cref="SerializationInfo"/> instance that defines the serialized data</param>
		/// <param name="context">A <see cref="StreamingContext"/> instance that contains the serialized data</param>
		[SecurityPermissionAttribute( SecurityAction.Demand, SerializationFormatter = true )]
		public virtual void GetObjectData( SerializationInfo info, StreamingContext context )
		{
			info.AddValue( "schema", schema );
			info.AddValue( "base", _base );
		}
	#endregion

	#region Methods
		/// <summary>
		/// Determine the <see cref="Axis"/> from which the <see cref="Bubble"/> charts are based.
		/// </summary>
		/// <seealso cref="ZedGraph.BubbleBase"/>
		/// <seealso cref="BubbleSettings"/>
		/// <seealso cref="ZedGraph.BubbleSettings.Base"/>
		/// <seealso cref="Scale.GetClusterWidth(GraphPane)"/>
		/// <returns>The <see cref="Axis"/> class for the axis from which the bubbles are based</returns>
		public Axis BubbleBaseAxis()
		{
			Axis bubbleAxis;
			if ( _base == BubbleBase.Y )
                bubbleAxis = _ownerPane.YAxis;
            else if (_base == BubbleBase.Y2)
                bubbleAxis = _ownerPane.Y2Axis;
            else if (_base == BubbleBase.X2)
                bubbleAxis = _ownerPane.X2Axis;
			else
                bubbleAxis = _ownerPane.XAxis;

            return bubbleAxis;
		}

	#endregion

	#region Defaults

		/// <summary>
		/// A simple struct that defines the
		/// default property values for the <see cref="BubbleSettings"/> class.
		/// </summary>
		public struct Default
		{
			/// <summary>The default value for the <see cref="BubbleSettings.Base"/>, which determines the base
			/// <see cref="Axis"/> from which the <see cref="Bubble"/> graphs will be displayed.
			/// </summary>
			/// <seealso cref="BubbleSettings.Base"/>
			public static BubbleBase Base = BubbleBase.X;
		}
	#endregion

    }
}
